#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const commander_1 = require("commander");
const promises_1 = __importDefault(require("fs/promises"));
const compiler_1 = require("../compiler");
const parse_1 = require("../interpreter/parse");
const DirectInterpreter_1 = require("../interpreter/DirectInterpreter");
const optimize_1 = require("../interpreter/optimize");
const inquirer_1 = __importDefault(require("inquirer"));
const base_1 = __importDefault(require("inquirer/lib/prompts/base"));
const program = new commander_1.Command();
const prepare = async (path, options) => {
    const content = await promises_1.default.readFile(path).then((c) => c.toString());
    const parsed = parse_1.parse(options.compiler ? compiler_1.compile(compiler_1.parse(content)).code : content);
    if (options.optimizer) {
        optimize_1.optimize(parsed);
    }
    if (options.benchmark)
        console.time("benchmark");
    return {
        parsed,
        output: (result) => {
            if (options.benchmark)
                console.timeEnd("benchmark");
            if (options.out) {
                promises_1.default.writeFile(options.out, result);
            }
            else {
                console.log(result);
            }
        },
    };
};
// const handle =
//   (handler: (statements: Statements, options: any) => string) =>
//   async (path: string, options: any) => {
//     console.log(program.opts());
//     const content = await fs.readFile(path).then((c) => c.toString());
//     const parsed = parseBrainfuck(
//       options.compiler ? compileToBrainfuck(parse(content)).code : content
//     );
//     if (options.optimizer) {
//       optimizeBrainfuck(parsed);
//     }
//     if (options.benchmark) console.time("benchmark");
//     const result = handler(parsed, options);
//     if (options.benchmark) console.timeEnd("benchmark");
//     if (options.out) {
//       fs.writeFile(options.out, result);
//     } else {
//       console.log(result);
//     }
//   };
inquirer_1.default.registerPrompt("char", class extends base_1.default {
    constructor(question, readLine, answers) {
        super(question, readLine, answers);
    }
    onKeyPress() {
        console.log("key press!");
    }
});
program
    .command("compile <path>")
    .option("-o, --out <outpath>", "output file path")
    .action(async (path, { out }) => {
    const content = await promises_1.default.readFile(path).then((c) => c.toString());
    const compiled = compiler_1.compile(compiler_1.parse(content)).code;
    if (out) {
        promises_1.default.writeFile(out, compiled);
    }
    else {
        console.log(compiled);
    }
});
program
    .command("run <path>")
    .option("-r, --no-compiler", "run raw brainfuck code")
    .option("-s, --no-optimizer", "run raw brainfuck code")
    .option("-n, --number <numbers...>", "specify numbers")
    .option("--benchmark", "show how long this action took")
    .action(async (path, options) => {
    const { parsed, output } = await prepare(path, options);
    await new DirectInterpreter_1.DirectInterpreter([]).interpret(parsed);
    output("");
}
// handle(async (statements, options) => {
//   console.log(options.opts());
//   await new DirectInterpreter(options.input?.map(parseInt)).interpret(
//     statements
//   );
//   return "";
// })
);
// program.command("tinker").action(async () => {
//   const interpreter = new DirectInterpreter([]);
//   while (true) {
//     const { code } = await inquirer.prompt([{ name: "code", message: ">" }]);
//     interpreter.interpret(parseBrainfuck(code));
//     console.log(
//       " ",
//       String.fromCharCode((interpreter as any).currentValue),
//       "(" + (interpreter as any).currentValue + ")"
//     );
//   }
// });
// program
//   .command("translate <path>")
//   .option("-o, --out <file>", "output file")
//   .option("-r, --no-compiler", "run raw brainfuck code")
//   .option("-s, --no-optimizer", "run raw brainfuck code")
//   .option(
//     "-l, --language <language>",
//     "the language to that the code should be translated"
//   )
//   .option("--benchmark", "show how long this action took")
//   .action(
//     handle((statements, options) => {
//       if (options.language == "c++") {
//         return new CTranslator().translate(statements);
//       }
//       throw new Error("Language not supported!");
//     })
//   );
program.parse(process.argv);
