"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = exports.compile = void 0;
const nearley_1 = __importDefault(require("nearley"));
const brainunfuck_js_1 = __importDefault(require("../brainunfuck.js"));
const generator_1 = require("./generator");
const astToBrainfuck = (environment, vars) => {
    const astToBrainfuck = (ast) => {
        if (ast.type == "int") {
            return new generator_1.Int(environment).setInt(ast.value);
        }
        if (ast.type == "char") {
            return new generator_1.Char(environment).setChar(ast.value);
        }
        if (ast.type == "string") {
            return new generator_1.BString(environment, ast.value);
        }
        if (ast.type == "additive_expression") {
            if (ast.operation == "+") {
                return astToBrainfuck(ast.left).clone().add(astToBrainfuck(ast.right));
            }
            if (ast.operation == "-") {
                return astToBrainfuck(ast.left)
                    .clone()
                    .subtract(astToBrainfuck(ast.right));
            }
        }
        if (ast.type == "read") {
            return new generator_1.Field(environment).input();
        }
        if (ast.type == "print_statement") {
            return astToBrainfuck(ast.value).print();
        }
        if (ast.type == "if_statement") {
            environment.executeIf(astToBrainfuck(ast.condition), () => {
                astToBrainfuck(ast.code);
            });
            return null;
        }
        if (ast.type == "while_statement") {
            environment.executeWhile(astToBrainfuck(ast.condition), () => {
                astToBrainfuck(ast.code);
            });
            return null;
        }
        if (ast.type == "code_block") {
            for (const statement of ast.value) {
                astToBrainfuck(statement);
            }
            return null;
        }
        if (ast.type == "var_initialisation") {
            if (ast.var_type == "int") {
                vars.set(ast.variable.name, new generator_1.Int(environment));
            }
            if (ast.var_type == "char") {
                vars.set(ast.variable.name, new generator_1.Char(environment));
            }
            return null;
        }
        if (ast.type == "var_assignment") {
            astToBrainfuck(ast.value).copy(vars.get(ast.variable.name));
            return null;
        }
        if (ast.type == "var_reference") {
            return vars.get(ast.name);
        }
    };
    return astToBrainfuck;
};
const compile = (ast) => {
    const vars = new Map();
    const environment = new generator_1.Environment();
    astToBrainfuck(environment, vars)(ast);
    return environment;
};
exports.compile = compile;
const parse = (code) => {
    const parser = new nearley_1.default.Parser(nearley_1.default.Grammar.fromCompiled(brainunfuck_js_1.default));
    parser.feed(code);
    return parser.results[0];
};
exports.parse = parse;
// console.log(code.value);
// interpret(code.value, ["A".charCodeAt()]); // 2 -> 1 (A)
// console.log();
// interpret(code.value, ["B".charCodeAt()]); // 3 -> 2 (B)
// console.log();
// interpret(code.value, ["C".charCodeAt()]); // 4 -> 3 (C)
// console.log();
// interpret(code.value, ["D".charCodeAt()]); // 5 -> 5 (E)
// console.log();
// interpret(code.value, ["E".charCodeAt()]); // 6 -> 8 (H)
// console.log();
// interpret(code.value, ["F".charCodeAt()]); // 7 -> 13 (M)
